      PROGRAM TBMHRP
C
C     tbmhrp writes a table of millstone hill local ionosphere model
C     parameter values.
C
C     Usage - tbmhrp pdir dayno1 dayno2 daynod lt1 lt2 ltd ...
C                    z1 z2 zd f107 ap ip tabfile
C        pdir     - directory containing model parameter files
C        dayno1   - start day number (january 1 = 1)
C        dayno2   - end day number
C        daynod   - day number increment
C        lt1      - start local time (hours)
C        lt2      - end local time
C        ltd      - local time increment
C        z1       - start altitude (km)
C        z2       - end altitude
C        zd       - altitude increment
C        f107     - 10.7 cm solar flux
C        ap       - 3-hour ap index
C        ip       - parameter code
c                      520 - log10 electron density
c                      550 - ion temperature (k)
c                      560 - electron temperature (k)
c                      580 - upward field aligned ion velocity (m/s)
C        tabfile  - output file name
C
C     e.g.: tbmhrp '/usr/oasis/experiments/madrigal' 1 1 1 12 12 12 ...
C                 100 1000 100 35 15 550 tbmhrp.out
C
C     .. Local Scalars ..
      DOUBLE PRECISION AP,DAYNO,DAYNO1,DAYNO2,DAYNOD,F107,LT,LT1,LT2,
     *                 LTD,Y,Z,Z1,Z2,ZD
      INTEGER I,IP,ISTAT,J,K,NDAYNO,NLT,NZ,STATUS
      CHARACTER*128 ARG,PDIR,TABFILE
C     ..
C     .. External Functions ..
      DOUBLE PRECISION GTMHRP
      INTEGER IARGC
      EXTERNAL GTMHRP,IARGC
C     ..
C     .....GET COMMAND LINE ARGUMENTS.....
C     .. External Subroutines ..
      EXTERNAL GETARG
C     ..
C
C     .. Intrinsic Functions ..
      INTRINSIC LEN
C     ..
      IF (IARGC().NE.14) THEN
	write(*,*)'------------------------------------------------------'
         WRITE (6,FMT=
     *'(''Usage: tbmhrp pdir dayno1 dayno2 daynod lt1 lt2 ''
     *    ''ltd z1 z2 zd f107 ap ip tabfile'')')
	write(*,*)'------------------------------------------------------'
	write(6,*)' tbmhrp: this executable file to generate model values'
	write(*,*)'   pdir: coefficient file directory; "." for current ',
     *                                                       'directory' 
	write(*,*)' dayno1: starting day number'
	write(*,*)' dayno2: ending day number'
	write(*,*)' daynod: day number increment(>0)'
	write(*,*)'    lt1: starting local time (hour)'
	write(*,*)'    lt2: ending local time (hour)'
	write(*,*)'    ltd: local time increment(hour,>0)'
	write(*,*)'     z1: starting altitude (km)'
	write(*,*)'     z2: ending altitude (km)'
	write(*,*)'     zd: altitude increment (km, >0)'
	write(*,*)'   f107: 10.7 cm solar flux for the previous day'
	write(*,*)'     ap: ap index for the previous 3 hours'
	write(*,*)'     ip: parameter code (CEDAR code)'
	write(*,*)'         520 = LOG10(Ne)'
	write(*,*)'         550 = Ti (K)'
	write(*,*)'         560 = Te (K)'
	write(*,*)'         580 = upward field aligned ion velocity (m/s)'
	write(*,*)'tabfile: output table file'
         STOP
      END IF
C
      CALL GETARG(1,PDIR)
      CALL GETARG(2,ARG)
      READ (UNIT=ARG,FMT=*) DAYNO1
      CALL GETARG(3,ARG)
      READ (UNIT=ARG,FMT=*) DAYNO2
      CALL GETARG(4,ARG)
      READ (UNIT=ARG,FMT=*) DAYNOD
      CALL GETARG(5,ARG)
      READ (UNIT=ARG,FMT=*) LT1
      CALL GETARG(6,ARG)
      READ (UNIT=ARG,FMT=*) LT2
      CALL GETARG(7,ARG)
      READ (UNIT=ARG,FMT=*) LTD
      CALL GETARG(8,ARG)
      READ (UNIT=ARG,FMT=*) Z1
      CALL GETARG(9,ARG)
      READ (UNIT=ARG,FMT=*) Z2
      CALL GETARG(10,ARG)
      READ (UNIT=ARG,FMT=*) ZD
      CALL GETARG(11,ARG)
      READ (UNIT=ARG,FMT=*) F107
      CALL GETARG(12,ARG)
      READ (UNIT=ARG,FMT=*) AP
      CALL GETARG(13,ARG)
      READ (UNIT=ARG,FMT=*) IP
      CALL GETARG(14,TABFILE)
C
      DO 10 I = 1,LEN(TABFILE)
         IF (TABFILE(I:I).NE.' ') THEN
            LT = I
         END IF
   10 CONTINUE
      OPEN (UNIT=16,FILE=TABFILE(1:LT),IOSTAT=STATUS)
      IF (STATUS.NE.0) THEN
         WRITE (6,FMT='(''Error opening output file'')')
         STOP
      END IF
      NDAYNO = (DAYNO2-DAYNO1)/DAYNOD + 1
      NLT = (LT2-LT1)/LTD + 1
      NZ = (Z2-Z1)/ZD + 1
C
      DAYNO = DAYNO1
      DO 40 K = 1,NDAYNO
         LT = LT1
         DO 30 J = 1,NLT
            Z = Z1
            DO 20 I = 1,NZ
               ISTAT = 0
               Y = GTMHRP(PDIR,DAYNO,LT,Z,F107,AP,IP,ISTAT)
               IF (IP.EQ.550 .OR. IP.EQ.560 .OR. IP.EQ.580) THEN
                  WRITE (16,FMT='(2f8.4,3f8.1,f8.1)') DAYNO,LT,Z,F107,
     *              AP,Y
               ELSE
                  WRITE (16,FMT='(2f8.4,3f8.1,f8.4)') DAYNO,LT,Z,F107,
     *              AP,Y
               END IF
               Z = Z + ZD
   20       CONTINUE
            LT = LT + LTD
   30    CONTINUE
         DAYNO = DAYNO + DAYNOD
   40 CONTINUE
      STOP
      END
      DOUBLE PRECISION FUNCTION GTMHRP(PDIR,DAYNO,LT,Z,F107,AP,IP,ISTAT)
C
C     GTMHRP RETURNS A MILLSTONE HILL LOCAL IONOSPHERE MODEL PARAMETER
C     VALUE.
C
C     PDIR  - DIRECTORY CONTAINING MODEL PARAMETER FILES
C     DAYNO - DAY NUMBER (JANUARY 1 = 1)
C     LT    - LOCAL TIME (HOURS)
C     Z     - ALTITUDE (KM)
C     F107  - 10.7 CM SOLAR FLUX
C     AP    - 3-HOUR AP INDEX
C     IP    - CEDAR CODE OF MODEL PARAMETER
C                520 - LOG10 ELECTRON DENSITY
C                550 - ION TEMPERATURE (K)
C                560 - ELECTRON TEMPERATURE (K)
C                580 - UPWARD FIELD ALIGNED ION VELOCITY (M/S)
C     ISTAT  - INPUT - 1 -> DIAGNOSTIC OUTPUT
C              INPUT - 0 -> SUCCESSFUL
C                      1 -> ILLEGAL PARAMETER CODE
C                      2 -> ALTITUDE OUT OF RANGE
C                      11 -> UNABLE TO OPEN AVTAB PARAMETER FILE
C                      12 -> UNABLE TO OPEN APTAB PARAMETER FILE
C                      13 -> UNABLE TO OPEN F107TAB PARAMETER FILE
C
C     .. Scalar Arguments ..
      DOUBLE PRECISION AP,DAYNO,F107,LT,Z
      INTEGER IP,ISTAT
      CHARACTER*(*) PDIR
C     ..
C     .. Common blocks ..
      COMMON /PARCOM/ZT,FYEAR,LTTAB,AVTAB,APTAB,F107TAB,NALTS,NLTS,
     *       NSEASONS,MONAME
      INTEGER NALTS,NLTS,NSEASONS
      DOUBLE PRECISION APTAB(16,24,12,4),AVTAB(16,24,12,4),
     *                 F107TAB(16,24,12,4),FYEAR(24,12,4),
     *                 LTTAB(24,12,4),ZT(16)
      CHARACTER*9 MONAME(12)
C     ..
C     .. Local Scalars ..
      DOUBLE PRECISION APN,F107N,FY,FYP,RJ1,RJ2,RK1,RK2
      INTEGER I,I1,I2,IDIAG,IX,J,J1,J2,K,K1,K2,KJ,PREAD
C     ..
C     .. Save statement ..
      SAVE PREAD
C     ..
C     .. Local Arrays ..
      DOUBLE PRECISION FI(2),FJ(2),FK(2)
      INTEGER II(2),JJ(2),KK(2)
      CHARACTER*3 PNAME(4)
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC DMOD
C     ..
C     .. External Subroutines ..
      EXTERNAL RDMHRP
C     ..
C     .. Data statements ..
      DATA PREAD/0/
      DATA PNAME/'Nel','Ti ','Te ','Vo '/
C     ..
C
      IDIAG = ISTAT
      ISTAT = 0
      IF (IP.EQ.520) THEN
         IX = 1
      ELSE IF (IP.EQ.550) THEN
         IX = 2
      ELSE IF (IP.EQ.560) THEN
         IX = 3
      ELSE IF (IP.EQ.580) THEN
         IX = 4
      ELSE
         ISTAT = 1
         GTMHRP = -9999.9D0
         RETURN
      END IF
C
C        .....READ THE MODEL TABLES IF NECESSARY.....
      IF (PREAD.EQ.0) THEN
         CALL RDMHRP(PDIR,ISTAT)
         IF (ISTAT.NE.0) THEN
            ISTAT = 10 + ISTAT
            GTMHRP = -9999.9D0
            RETURN
         END IF
         PREAD = 1
      END IF
C
C     ....NORMALIZE AP AND F10.7.....
      APN = (AP-15.0D0)/10.0D0
      F107N = (F107-135.0D0)/100.0D0
C
C     .....GET POSITION OF POINT IN ALTITUDE.....
      DO 10 I = 1,NALTS - 1
         IF (Z.GE.ZT(I) .AND. Z.LE.ZT(I+1)) THEN
            I1 = I
            GO TO 20
         END IF
   10 CONTINUE
      ISTAT = 2
      GTMHRP = -9999.9D0
      RETURN
   20 CONTINUE
      I2 = I1 + 1
      II(1) = I1
      II(2) = I2
      FI(1) = (ZT(I2)-Z)/(ZT(I2)-ZT(I1))
      FI(2) = (Z-ZT(I1))/(ZT(I2)-ZT(I1))
      IF (IDIAG.EQ.1) THEN
         WRITE (6,FMT='(f8.1,2x,2f8.1,2x,2f8.1)') Z,ZT(I1),ZT(I2),FI(1),
     *     FI(2)
      END IF
C
C     .....GET POSITION OF POINT IN LOCAL TIME.....
      FY = DMOD(LT,24.0D0)/24.0D0
      FYP = FY + 0.5D0/24.0D0
      FYP = DMOD(FYP,1.0D0)
      IF (FYP.LT.0.0D0) FYP = FYP + 1.0D0
      J1 = 24.0D0*FYP
      IF (J1.EQ.0) J1 = 24
      J2 = J1 + 1
      IF (J2.EQ.25) J2 = 1
      RJ1 = (J1-0.5D0)/24.0D0
      IF (FY.LT.RJ1 .AND. J1.EQ.24) RJ1 = RJ1 - 1.0D0
      IF (KJ.EQ.13) J2 = 1
      RJ2 = (J2-0.5D0)/24.0D0
      IF (FY.GT.RJ2 .AND. J2.EQ.1) RJ2 = RJ2 + 1.0D0
      JJ(1) = J1
      JJ(2) = J2
      FJ(1) = (RJ2-FY)/(RJ2-RJ1)
      FJ(2) = (FY-RJ1)/(RJ2-RJ1)
      IF (IDIAG.EQ.1) THEN
         WRITE (6,FMT='(2F8.4,2I4,2X,2F8.4,2X,f8.3,2X,2f8.4)') LT,FY,J1,
     *     J2,RJ1,RJ2,RJ2 - RJ1,FJ(1),FJ(2)
      END IF
C
C     ....GET POSITION OF POINT IN SEASON.....
      FY = DMOD(DAYNO,365.0D0)/365.0D0
      FYP = FY + 0.5D0/12.0D0
      FYP = DMOD(FYP,1.0D0)
      IF (FYP.LT.0.0D0) FYP = FYP + 1.0D0
      K1 = 12.0D0*FYP
      IF (K1.EQ.0) K1 = 12
      K2 = K1 + 1
      IF (K2.EQ.13) K2 = 1
      RK1 = (K1-0.5D0)/12.0D0
      IF (FY.LT.RK1 .AND. K1.EQ.12) RK1 = RK1 - 1.0D0
      IF (K2.EQ.13) K2 = 1
      RK2 = (K2-0.5D0)/12.0D0
      IF (FY.GT.RK2 .AND. K2.EQ.1) RK2 = RK2 + 1.0D0
      KK(1) = K1
      KK(2) = K2
      FK(1) = (RK2-FY)/(RK2-RK1)
      FK(2) = (FY-RK1)/(RK2-RK1)
      IF (IDIAG.EQ.1) THEN
         WRITE (6,FMT='(2F8.4,2I4,2X,2F8.4,2X,2X,f8.4,2X,2f8.4)') DAYNO,
     *     FY,K1,K2,RK1,RK2,RK2 - RK1,FK(1),FK(2)
      END IF
C
C     .....EVALUATE PARAMETER.....
      GTMHRP = 0.0D0
      DO 50 K = 1,2
         DO 40 J = 1,2
            DO 30 I = 1,2
               IF (IDIAG.EQ.1) THEN
                  WRITE (6,FMT='(4X,6i3,4e13.5)') I,J,K,II(I),JJ(J),
     *              KK(K),FI(I),FJ(J),FK(K),AVTAB(II(I),JJ(J),KK(K),IX)
               END IF
               GTMHRP = GTMHRP + FI(I)*FJ(J)*FK(K)*
     *                  (AVTAB(II(I),JJ(J),KK(K),IX)+
     *                  APN*APTAB(II(I),JJ(J),KK(K),IX)+
     *                  F107N*F107TAB(II(I),JJ(J),KK(K),IX))
   30       CONTINUE
   40    CONTINUE
   50 CONTINUE
C
      RETURN
      END
      SUBROUTINE RDMHRP(PDIR,ISTAT)
C
C     RDMHRP READS THE MILLSTONE HILL LOCAL IONOSPHERE MODEL
C     PARAMETER FILES.
C
C     PDIR  - DIRECTORY CONTAINING MODEL PARAMETER FILES
C     ISTAT - 0 - SUCCESSFUL
C             1 -> UNABLE TO OPEN AVTAB PARAMETER FILE
C             2 -> UNABLE TO OPEN APTAB PARAMETER FILE
C             3 -> UNABLE TO OPEN F107TAB PARAMETER FILE
C
C     .. Scalar Arguments ..
      INTEGER ISTAT
      CHARACTER*(*) PDIR
C     ..
C     .. Common blocks ..
      COMMON /PARCOM/ZT,FYEAR,LTTAB,AVTAB,APTAB,F107TAB,NALTS,NLTS,
     *       NSEASONS,MONAME
      INTEGER NALTS,NLTS,NSEASONS
      DOUBLE PRECISION APTAB(16,24,12,4),AVTAB(16,24,12,4),
     *                 F107TAB(16,24,12,4),FYEAR(24,12,4),
     *                 LTTAB(24,12,4),ZT(16)
      CHARACTER*9 MONAME(12)
C     ..
C     .. Local Scalars ..
      INTEGER I,IX,J,K,LD,LN,PARCODE,PREAD,STATUS
      CHARACTER*128 FILNAM
C     ..
C     .. Local Arrays ..
      CHARACTER*3 PNAME(4)
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC LEN
C     ..
C     .. Data statements ..
      DATA PREAD/0/
      DATA PNAME/'Nel','Ti ','Te ','Vo '/
C     ..
C
      ISTAT = 0
C
C        .....READ THE MODEL TABLES.....
      DO 10 I = 1,LEN(PDIR)
         IF (PDIR(I:I).NE.' ') THEN
            LD = I
         END IF
   10 CONTINUE
C
      DO 90 IX = 1,3
         DO 20 I = 1,LEN(PNAME(IX))
            IF (PNAME(IX) (I:I).NE.' ') THEN
               LN = I
            END IF
   20    CONTINUE
         FILNAM = PDIR(1:LD)//'/avtab'//PNAME(IX) (1:LN)//'.dat'
         OPEN (UNIT=15,FILE=FILNAM,IOSTAT=STATUS)
         IF (STATUS.NE.0) THEN
            ISTAT = 1
            RETURN
         END IF
         READ (15,FMT='(I4)') PARCODE
         READ (15,FMT='(3I4)') NALTS,NLTS,NSEASONS
         READ (15,FMT='(16F8.1)') (ZT(I),I=1,NALTS)
         DO 40 K = 1,12
            READ (15,FMT='(A)') MONAME(K)
            READ (15,FMT='(24F8.4)') (FYEAR(J,K,IX),J=1,24)
            READ (15,FMT='(24F8.4)') (LTTAB(J,K,IX),J=1,24)
            DO 30 I = 16,1,-1
               READ (15,FMT=*) (AVTAB(I,J,K,IX),J=1,24)
   30       CONTINUE
   40    CONTINUE
         CLOSE (15)
C
         FILNAM = PDIR(1:LD)//'/aptab'//PNAME(IX) (1:LN)//'.dat'
         OPEN (UNIT=15,FILE=FILNAM,IOSTAT=STATUS)
         IF (STATUS.NE.0) THEN
            ISTAT = 2
            RETURN
         END IF
         READ (15,FMT='(I4)') PARCODE
         READ (15,FMT='(3I4)') NALTS,NLTS,NSEASONS
         READ (15,FMT='(16F8.1)') (ZT(I),I=1,NALTS)
         DO 60 K = 1,12
            READ (15,FMT='(A)') MONAME(K)
            READ (15,FMT='(24F8.4)') (FYEAR(J,K,IX),J=1,24)
            READ (15,FMT='(24F8.4)') (LTTAB(J,K,IX),J=1,24)
            DO 50 I = 16,1,-1
               READ (15,FMT=*) (APTAB(I,J,K,IX),J=1,24)
c                 WRITE (6,FMT='(3i4,f8.1)') I,K,IP,ApTAB(I,1,K,IiX)
   50       CONTINUE
   60    CONTINUE
         CLOSE (15)
C
         FILNAM = PDIR(1:LD)//'/f107tab'//PNAME(IX) (1:LN)//'.dat'
         OPEN (UNIT=15,FILE=FILNAM,IOSTAT=STATUS)
         IF (STATUS.NE.0) THEN
            ISTAT = 3
            RETURN
         END IF
         READ (15,FMT='(I4)') PARCODE
         READ (15,FMT='(3I4)') NALTS,NLTS,NSEASONS
         READ (15,FMT='(16F8.1)') (ZT(I),I=1,NALTS)
         DO 80 K = 1,12
            READ (15,FMT='(A)') MONAME(K)
            READ (15,FMT='(24F8.4)') (FYEAR(J,K,IX),J=1,24)
            READ (15,FMT='(24F8.4)') (LTTAB(J,K,IX),J=1,24)
            DO 70 I = 16,1,-1
               READ (15,FMT=*) (F107TAB(I,J,K,IX),J=1,24)
   70       CONTINUE
   80    CONTINUE
         CLOSE (15)
   90 CONTINUE
C
      RETURN
      END

